@@ -89,6 +89,7 @@ class DatabaseMappingBase:
         self._object_entity_type = None
         self._relationship_entity_type = None
         # Subqueries that select everything from each table
+        self._commit_sq = None
         self._alternative_sq = None
         self._scenario_sq = None
         self._scenario_alternative_sq = None
@@ -103,8 +104,6 @@ class DatabaseMappingBase:
         self._entity_group_sq = None
         self._parameter_definition_sq = None
         self._parameter_value_sq = None
-        self._parameter_tag_sq = None
-        self._parameter_definition_tag_sq = None
         self._parameter_value_list_sq = None
         self._feature_sq = None
         self._tool_sq = None
@@ -130,8 +129,6 @@ class DatabaseMappingBase:
         self._entity_parameter_value_sq = None
         self._object_parameter_value_sq = None
         self._relationship_parameter_value_sq = None
-        self._ext_parameter_definition_tag_sq = None
-        self._wide_parameter_definition_tag_sq = None
         self._wide_parameter_value_list_sq = None
         self._ext_feature_sq = None
         self._ext_tool_feature_sq = None
@@ -154,6 +151,76 @@ class DatabaseMappingBase:
             "relationship_entity": ("entity_id", "dimension"),
             "relationship_entity_class": ("entity_class_id", "dimension"),
         }
+        # Subqueries used to populate cache
+        self.cache_sqs = {
+            "feature": "ext_feature_sq",
+            "tool": "tool_sq",
+            "tool_feature": "ext_tool_feature_sq",
+            "tool_feature_method": "tool_feature_method_sq",
+            "parameter_value_list": "wide_parameter_value_list_sq",
+            "alternative": "alternative_sq",
+            "scenario": "wide_scenario_sq",
+            "scenario_alternative": "scenario_alternative_sq",
+            "object_class": "object_class_sq",
+            "object": "ext_object_sq",
+            "relationship_class": "wide_relationship_class_sq",
+            "relationship": "wide_relationship_sq",
+            "entity_group": "ext_entity_group_sq",
+            "parameter_definition": "entity_parameter_definition_sq",
+            "object_parameter_definition": "object_parameter_definition_sq",
+            "relationship_parameter_definition": "relationship_parameter_definition_sq",
+            "parameter_value": "entity_parameter_value_sq",
+            "object_parameter_value": "object_parameter_value_sq",
+            "relationship_parameter_value": "relationship_parameter_value_sq",
+            "metadata": "metadata_sq",
+            "entity_metadata": "entity_metadata_sq",
+            "parameter_value_metadata": "parameter_value_metadata_sq",
+        }
+        self.ancestor_tablenames = {
+            "feature": ("parameter_definition",),
+            "tool_feature": ("tool", "feature"),
+            "tool_feature_method": ("tool_feature", "parameter_value_list"),
+            "scenario_alternative": ("scenario", "alternative"),
+            "relationship_class": ("object_class",),
+            "object": ("object_class",),
+            "entity_group": ("object_class", "relationship_class", "object", "relationship"),
+            "relationship": ("relationship_class", "object"),
+            "parameter_definition": ("object_class", "relationship_class", "parameter_value_list"),
+            "parameter_value": (
+                "alternative",
+                "object_class",
+                "relationship_class",
+                "object",
+                "relationship",
+                "parameter_definition",
+                "parameter_value_list",
+            ),
+            "entity_metadata": ("metadata",),
+            "parameter_value_metadata": ("metadata",),
+        }
+        self.descendant_tablenames = {
+            tablename: set(self._descendant_tablenames(tablename)) for tablename in self.cache_sqs
+        }
+
+    def _descendant_tablenames(self, tablename):
+        child_tablenames = {
+            "alternative": ("parameter_value", "scenario_alternative"),
+            "scenario": ("scenario_alternative",),
+            "object_class": ("object", "relationship_class", "parameter_definition"),
+            "object": ("relationship", "parameter_value", "entity_group"),
+            "relationship_class": ("relationship", "parameter_definition"),
+            "relationship": ("parameter_value", "entity_group"),
+            "parameter_definition": ("parameter_value", "feature"),
+            "parameter_value_list": ("feature",),
+            "feature": ("tool_feature",),
+            "tool": ("tool_feature",),
+            "tool_feature": ("tool_feature_method",),
+        }
+        for parent, children in child_tablenames.items():
+            if tablename == parent:
+                for child in children:
+                    yield child
+                    yield from self._descendant_tablenames(child)
 
     def make_commit_id(self):
         return None
@@ -329,7 +396,7 @@ class DatabaseMappingBase:
 
         .. code-block:: sql
 
-            SELECT * FROM {tablename}
+            SELECT * FROM tablename
 
         Args:
             tablename (str): the table to be queried.
@@ -603,36 +670,6 @@ class DatabaseMappingBase:
             self._parameter_value_sq = self._make_parameter_value_sq()
         return self._parameter_value_sq
 
-    @property
-    def parameter_tag_sq(self):
-        """A subquery of the form:
-
-        .. code-block:: sql
-
-            SELECT * FROM parameter_tag
-
-        Returns:
-            sqlalchemy.sql.expression.Alias
-        """
-        if self._parameter_tag_sq is None:
-            self._parameter_tag_sq = self._subquery("parameter_tag")
-        return self._parameter_tag_sq
-
-    @property
-    def parameter_definition_tag_sq(self):
-        """A subquery of the form:
-
-        .. code-block:: sql
-
-            SELECT * FROM parameter_definition_tag
-
-        Returns:
-            sqlalchemy.sql.expression.Alias
-        """
-        if self._parameter_definition_tag_sq is None:
-            self._parameter_definition_tag_sq = self._subquery("parameter_definition_tag")
-        return self._parameter_definition_tag_sq
-
     @property
     def parameter_value_list_sq(self):
         """A subquery of the form:
@@ -690,6 +727,12 @@ class DatabaseMappingBase:
             self._entity_metadata_sq = self._subquery("entity_metadata")
         return self._entity_metadata_sq
 
+    @property
+    def commit_sq(self):
+        if self._commit_sq is None:
+            self._commit_sq = self._subquery("commit")
+        return self._commit_sq
+
     @property
     def ext_scenario_sq(self):
         if self._ext_scenario_sq is None:
@@ -1056,13 +1099,11 @@ class DatabaseMappingBase:
                     self.parameter_definition_sq.c.name.label("parameter_name"),
                     self.parameter_definition_sq.c.parameter_value_list_id.label("value_list_id"),
                     self.wide_parameter_value_list_sq.c.name.label("value_list_name"),
-                    self.wide_parameter_definition_tag_sq.c.parameter_tag_id_list,
-                    self.wide_parameter_definition_tag_sq.c.parameter_tag_list,
                     self.parameter_definition_sq.c.default_value,
+                    self.parameter_definition_sq.c.default_type,
                     self.parameter_definition_sq.c.description,
                 )
                 .filter(self.entity_class_sq.c.id == self.parameter_definition_sq.c.entity_class_id)
-                .filter(self.wide_parameter_definition_tag_sq.c.id == self.parameter_definition_sq.c.id)
                 .outerjoin(
                     self.wide_parameter_value_list_sq,
                     self.wide_parameter_value_list_sq.c.id == self.parameter_definition_sq.c.parameter_value_list_id,
@@ -1084,25 +1125,8 @@ class DatabaseMappingBase:
                 pd.name AS parameter_name,
                 wpvl.id AS value_list_id,
                 wpvl.name AS value_list_name,
-                wpdt.parameter_tag_id_list,
-                wpdt.parameter_tag_list,
                 pd.default_value
             FROM parameter_definition AS pd, object_class AS oc
-            LEFT JOIN (
-                SELECT
-                    parameter_definition_id,
-                    GROUP_CONCAT(parameter_tag_id) AS parameter_tag_id_list,
-                    GROUP_CONCAT(parameter_tag) AS parameter_tag_list
-                FROM (
-                    SELECT
-                        pdt.parameter_definition_id,
-                        pt.id AS parameter_tag_id,
-                        pt.tag AS parameter_tag
-                    FROM parameter_definition_tag as pdt, parameter_tag AS pt
-                    WHERE pdt.parameter_tag_id = pt.id
-                )
-                GROUP BY parameter_definition_id
-            ) AS wpdt
             ON wpdt.parameter_definition_id = pd.id
             LEFT JOIN (
                 SELECT
@@ -1132,14 +1156,11 @@ class DatabaseMappingBase:
                     self.parameter_definition_sq.c.name.label("parameter_name"),
                     self.parameter_definition_sq.c.parameter_value_list_id.label("value_list_id"),
                     self.wide_parameter_value_list_sq.c.name.label("value_list_name"),
-                    self.wide_parameter_definition_tag_sq.c.parameter_tag_id_list,
-                    self.wide_parameter_definition_tag_sq.c.parameter_tag_list,
                     self.parameter_definition_sq.c.default_value,
                     self.parameter_definition_sq.c.default_type,
                     self.parameter_definition_sq.c.description,
                 )
                 .filter(self.object_class_sq.c.id == self.parameter_definition_sq.c.object_class_id)
-                .filter(self.wide_parameter_definition_tag_sq.c.id == self.parameter_definition_sq.c.id)
                 .outerjoin(
                     self.wide_parameter_value_list_sq,
                     self.wide_parameter_value_list_sq.c.id == self.parameter_definition_sq.c.parameter_value_list_id,
@@ -1163,8 +1184,6 @@ class DatabaseMappingBase:
                 pd.name AS parameter_name,
                 wpvl.id AS value_list_id,
                 wpvl.name AS value_list_name,
-                wpdt.parameter_tag_id_list,
-                wpdt.parameter_tag_list,
                 pd.default_value
             FROM
                 parameter_definition AS pd,
@@ -1186,21 +1205,6 @@ class DatabaseMappingBase:
                     )
                     GROUP BY id, name
                 ) AS wrc
-            LEFT JOIN (
-                SELECT
-                    parameter_definition_id,
-                    GROUP_CONCAT(parameter_tag_id) AS parameter_tag_id_list,
-                    GROUP_CONCAT(parameter_tag) AS parameter_tag_list
-                FROM (
-                    SELECT
-                        pdt.parameter_definition_id,
-                        pt.id AS parameter_tag_id,
-                        pt.tag AS parameter_tag
-                    FROM parameter_definition_tag as pdt, parameter_tag AS pt
-                    WHERE pdt.parameter_tag_id = pt.id
-                )
-                GROUP BY parameter_definition_id
-            ) AS wpdt
             ON wpdt.parameter_definition_id = pd.id
             LEFT JOIN (
                 SELECT
@@ -1232,14 +1236,11 @@ class DatabaseMappingBase:
                     self.parameter_definition_sq.c.name.label("parameter_name"),
                     self.parameter_definition_sq.c.parameter_value_list_id.label("value_list_id"),
                     self.wide_parameter_value_list_sq.c.name.label("value_list_name"),
-                    self.wide_parameter_definition_tag_sq.c.parameter_tag_id_list,
-                    self.wide_parameter_definition_tag_sq.c.parameter_tag_list,
                     self.parameter_definition_sq.c.default_value,
                     self.parameter_definition_sq.c.default_type,
                     self.parameter_definition_sq.c.description,
                 )
                 .filter(self.parameter_definition_sq.c.relationship_class_id == self.wide_relationship_class_sq.c.id)
-                .filter(self.wide_parameter_definition_tag_sq.c.id == self.parameter_definition_sq.c.id)
                 .outerjoin(
                     self.wide_parameter_value_list_sq,
                     self.wide_parameter_value_list_sq.c.id == self.parameter_definition_sq.c.parameter_value_list_id,
@@ -1318,7 +1319,6 @@ class DatabaseMappingBase:
         Returns:
             sqlalchemy.sql.expression.Alias
         """
-        # TODO: Should this also bring `value_list` and `tag_list`?
         if self._relationship_parameter_value_sq is None:
             self._relationship_parameter_value_sq = (
                 self.query(
@@ -1347,78 +1347,6 @@ class DatabaseMappingBase:
             )
         return self._relationship_parameter_value_sq
 
-    @property
-    def ext_parameter_definition_tag_sq(self):
-        """A subquery of the form:
-
-        .. code-block:: sql
-
-            SELECT
-                pdt.parameter_definition_id,
-                pt.id AS parameter_tag_id,
-                pt.tag AS parameter_tag
-            FROM parameter_definition_tag as pdt, parameter_tag AS pt
-            WHERE pdt.parameter_tag_id = pt.id
-
-        Returns:
-            sqlalchemy.sql.expression.Alias
-        """
-        if self._ext_parameter_definition_tag_sq is None:
-            self._ext_parameter_definition_tag_sq = (
-                self.query(
-                    self.parameter_definition_sq.c.id.label("parameter_definition_id"),
-                    self.parameter_definition_tag_sq.c.parameter_tag_id.label("parameter_tag_id"),
-                    self.parameter_tag_sq.c.tag.label("parameter_tag"),
-                )
-                .outerjoin(
-                    self.parameter_definition_tag_sq,
-                    self.parameter_definition_tag_sq.c.parameter_definition_id == self.parameter_definition_sq.c.id,
-                )
-                .outerjoin(
-                    self.parameter_tag_sq,
-                    self.parameter_tag_sq.c.id == self.parameter_definition_tag_sq.c.parameter_tag_id,
-                )
-                .subquery()
-            )
-        return self._ext_parameter_definition_tag_sq
-
-    @property
-    def wide_parameter_definition_tag_sq(self):
-        """A subquery of the form:
-
-        .. code-block:: sql
-
-            SELECT
-                parameter_definition_id,
-                GROUP_CONCAT(parameter_tag_id) AS parameter_tag_id_list,
-                GROUP_CONCAT(parameter_tag) AS parameter_tag_list
-            FROM (
-                SELECT
-                    pdt.parameter_definition_id,
-                    pt.id AS parameter_tag_id,
-                    pt.tag AS parameter_tag
-                FROM parameter_definition_tag as pdt, parameter_tag AS pt
-                WHERE pdt.parameter_tag_id = pt.id
-            )
-            GROUP BY parameter_definition_id
-
-        Returns:
-            sqlalchemy.sql.expression.Alias
-        """
-        if self._wide_parameter_definition_tag_sq is None:
-            self._wide_parameter_definition_tag_sq = (
-                self.query(
-                    self.ext_parameter_definition_tag_sq.c.parameter_definition_id.label("id"),
-                    group_concat(self.ext_parameter_definition_tag_sq.c.parameter_tag_id).label(
-                        "parameter_tag_id_list"
-                    ),
-                    group_concat(self.ext_parameter_definition_tag_sq.c.parameter_tag).label("parameter_tag_list"),
-                )
-                .group_by(self.ext_parameter_definition_tag_sq.c.parameter_definition_id)
-                .subquery()
-            )
-        return self._wide_parameter_definition_tag_sq
-
     @property
     def wide_parameter_value_list_sq(self):
         """A subquery of the form:
@@ -1794,6 +1722,120 @@ class DatabaseMappingBase:
                 msg = f"DBAPIError while removing {tablename} items: {e.orig.args}"
                 raise SpineDBAPIError(msg)
 
+    def make_cache(self, tablenames, only_descendants=False, include_ancestors=False):
+        if only_descendants:
+            tablenames = {
+                descendant for tablename in tablenames for descendant in self.descendant_tablenames.get(tablename, ())
+            }
+        if include_ancestors:
+            tablenames |= {
+                ancestor for tablename in tablenames for ancestor in self.ancestor_tablenames.get(tablename, ())
+            }
+        return {
+            tablename: {x.id: x for x in self.query(getattr(self, self.cache_sqs[tablename]))}
+            for tablename in tablenames & self.cache_sqs.keys()
+        }
+
+    @staticmethod
+    def cache_to_db(item_type, item):
+        """
+        Returns the db equivalent of a cache item.
+
+        Args:
+            item_type (str): The item type
+            item (dict): The item in the cache
+
+        Returns:
+            dict
+        """
+        if item_type == "relationship_class":
+            return DatabaseMappingBase.cache_relationship_class_to_db(item)
+        if item_type == "relationship":
+            return DatabaseMappingBase.cache_relationship_to_db(item)
+        if item_type == "parameter_definition":
+            return DatabaseMappingBase.cache_parameter_definition_to_db(item)
+        if item_type == "parameter_value":
+            return DatabaseMappingBase.cache_parameter_value_to_db(item)
+        if item_type == "parameter_value_list":
+            return DatabaseMappingBase.cache_parameter_value_list_to_db(item)
+        if item_type == "entity_group":
+            return DatabaseMappingBase.cache_entity_group_to_db(item)
+        return item.copy()
+
+    @staticmethod
+    def cache_relationship_class_to_db(item):
+        return {
+            "id": item["id"],
+            "name": item["name"],
+            "description": item.get("description"),
+            "object_class_id_list": [int(id_) for id_ in item["object_class_id_list"].split(",")],
+        }
+
+    @staticmethod
+    def cache_relationship_to_db(item):
+        return {
+            "id": item["id"],
+            "name": item["name"],
+            "class_id": item["class_id"],
+            "object_class_id_list": [int(id_) for id_ in item["object_class_id_list"].split(",")],
+            "object_id_list": [int(id_) for id_ in item["object_id_list"].split(",")],
+        }
+
+    @staticmethod
+    def cache_parameter_definition_to_db(item):
+        return {
+            "id": item["id"],
+            "entity_class_id": item["entity_class_id"],
+            "name": item["parameter_name"],
+            "parameter_value_list_id": item.get("value_list_id"),
+            "default_value": item.get("default_value"),
+            "default_type": item.get("default_type"),
+            "description": item.get("description"),
+        }
+
+    @staticmethod
+    def cache_parameter_value_to_db(item):
+        return {
+            "id": item["id"],
+            "entity_class_id": item["entity_class_id"],
+            "entity_id": item["entity_id"],
+            "parameter_definition_id": item["parameter_id"],
+            "alternative_id": item["alternative_id"],
+            "value": item["value"],
+            "type": item["type"],
+        }
+
+    @staticmethod
+    def cache_parameter_value_list_to_db(item):
+        return {
+            "id": item["id"],
+            "name": item["name"],
+            "value_list": [bytes(val, "UTF8") for val in item["value_list"].split(";")],
+        }
+
+    @staticmethod
+    def cache_entity_group_to_db(item):
+        return {
+            "id": item["id"],
+            "entity_class_id": item["class_id"],
+            "entity_id": item["group_id"],
+            "member_id": item["member_id"],
+        }
+
+    def _items_with_type_id(self, tablename, *items):
+        type_id = {
+            "object_class": self.object_class_type,
+            "relationship_class": self.relationship_class_type,
+            "object": self.object_entity_type,
+            "relationship": self.relationship_entity_type,
+        }.get(tablename)
+        if type_id is None:
+            yield from items
+            return
+        for item in items:
+            item["type_id"] = type_id
+            yield item
+
     def __del__(self):
         try:
             self.connection.close()
